@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2010 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -89,7 +89,10 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_NO_EMBEDDED_RDWR	, "Error : cannot open embedded file read/write." },
 	{	SFE_NO_PIPE_WRITE		, "Error : this file format does not support pipe write." },
 	{	SFE_BAD_VIRTUAL_IO		, "Error : bad pointer on SF_VIRTUAL_IO struct." },
-	{	SFE_BAD_BROADCAST_INFO_SIZE, "Error : badd SF_BROADCAST_INFO_SIZE." },
+	{	SFE_BAD_BROADCAST_INFO_SIZE
+								, "Error : bad size in SF_BROADCAST_INFO struct." },
+	{	SFE_BAD_BROADCAST_INFO_TOO_BIG
+								, "Error : SF_BROADCAST_INFO struct too large." },
 
 	{	SFE_INTERLEAVE_MODE		, "Attempt to write to file with non-interleaved data." },
 	{	SFE_INTERLEAVE_SEEK		, "Bad karma in seek during interleave read operation." },
@@ -237,7 +240,7 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_FLAC_INIT_DECODER	, "Error : problem while initialization of the flac decoder." },
 	{	SFE_FLAC_LOST_SYNC		, "Error : flac decoder lost sync." },
 	{	SFE_FLAC_BAD_SAMPLE_RATE, "Error : flac does not support this sample rate." },
-	{	SFE_FLAC_UNKOWN_ERROR	, "Error : unkown error in flac decoder." },
+	{	SFE_FLAC_UNKOWN_ERROR	, "Error : unknown error in flac decoder." },
 
 	{	SFE_WVE_NOT_WVE			, "Error : not a WVE file." },
 	{	SFE_WVE_NO_PIPE			, "Error : not able to operate on WVE files over a pipe." },
@@ -303,6 +306,9 @@ SNDFILE*
 sf_open	(const char *path, int mode, SF_INFO *sfinfo)
 {	SF_PRIVATE 	*psf ;
 
+	/* Ultimate sanity check. */
+	SF_ASSERT (sizeof (sf_count_t) == 8) ;
+
 	if ((psf = calloc (1, sizeof (SF_PRIVATE))) == NULL)
 	{	sf_errno = SFE_MALLOC_FAILED ;
 		return	NULL ;
@@ -641,16 +647,14 @@ sf_format_check	(const SF_INFO *info)
 				break ;
 
 		case SF_FORMAT_SVX :
-				/* SVX currently does not support more than one channel for write.
-				** Read will allow more than one channel but only allow one here.
-				*/
-				if (info->channels != 1)
+				/* SVX only supports writing mono SVX files. */
+				if (info->channels > 1)
 					return 0 ;
 				/* Always big endian. */
 				if (endian == SF_ENDIAN_LITTLE || endian == SF_ENDIAN_CPU)
 					return 0 ;
 
-				if ((subformat == SF_FORMAT_PCM_S8 || subformat == SF_FORMAT_PCM_16) && info->channels == 1)
+				if (subformat == SF_FORMAT_PCM_S8 || subformat == SF_FORMAT_PCM_16)
 					return 1 ;
 				break ;
 
@@ -887,7 +891,7 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 
 	if (sndfile == NULL && command == SFC_GET_LOG_INFO)
 	{	if (data == NULL)
-			return (psf->error = SFE_BAD_COMMAND_PARAM) ;
+			return (sf_errno = SFE_BAD_COMMAND_PARAM) ;
 		snprintf (data, datasize, "%s", sf_logbuffer) ;
 		return strlen (data) ;
 		} ;
@@ -1060,6 +1064,11 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 				return SF_TRUE ;
 			if (datasize != sizeof (sf_count_t))
 				return SF_TRUE ;
+			if (data == NULL || datasize != sizeof (sf_count_t))
+			{	psf->error = SFE_BAD_COMMAND_PARAM ;
+				return SF_FALSE ;
+				}
+			else
 			{	sf_count_t position ;
 
 				position = *((sf_count_t*) data) ;
@@ -1135,7 +1144,7 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 			if ((psf->file.mode != SFM_WRITE) && (psf->file.mode != SFM_RDWR))
 				return SF_FALSE ;
 			/* If data has already been written this must fail. */
-			if (psf->broadcast_var == NULL && psf->have_written)
+			if (psf->broadcast_16k == NULL && psf->have_written)
 			{	psf->error = SFE_CMD_HAS_DATA ;
 				return SF_FALSE ;
 				} ;
@@ -1419,7 +1428,7 @@ sf_read_raw		(SNDFILE *sndfile, void *ptr, sf_count_t bytes)
 		return	0 ;
 		} ;
 
-	if (bytes < 0 || psf->read_current >= psf->datalength)
+	if (bytes < 0 || psf->read_current >= psf->sf.frames)
 	{	psf_memset (ptr, 0, bytes) ;
 		return 0 ;
 		} ;
@@ -1429,6 +1438,10 @@ sf_read_raw		(SNDFILE *sndfile, void *ptr, sf_count_t bytes)
 		return 0 ;
 		} ;
 
+	if (psf->last_op != SFM_READ)
+		if (psf->seek (psf, SFM_READ, psf->read_current) < 0)
+			return 0 ;
+
 	count = psf_fread (ptr, 1, bytes, psf) ;
 
 	if (psf->read_current + count / blockwidth <= psf->sf.frames)
@@ -1837,6 +1850,10 @@ sf_write_raw	(SNDFILE *sndfile, const void *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
+	if (psf->last_op != SFM_WRITE)
+		if (psf->seek (psf, SFM_WRITE, psf->write_current) < 0)
+			return 0 ;
+
 	if (psf->have_written == SF_FALSE && psf->write_header != NULL)
 		psf->write_header (psf, SF_FALSE) ;
 	psf->have_written = SF_TRUE ;
@@ -1845,11 +1862,14 @@ sf_write_raw	(SNDFILE *sndfile, const void *ptr, sf_count_t len)
 
 	psf->write_current += count / blockwidth ;
 
+	psf->last_op = SFM_WRITE ;
+
+	if (psf->auto_header && psf->write_header != NULL)
+		psf->write_header (psf, SF_TRUE) ;
+
 	if (psf->write_current > psf->sf.frames)
 		psf->sf.frames = psf->write_current ;
 
-	psf->last_op = SFM_WRITE ;
-
 	return count ;
 } /* sf_write_raw */
 
@@ -2379,23 +2399,31 @@ guess_file_type (SF_PRIVATE *psf)
 	if (buffer [0] == MAKE_MARKER ('f', 'L', 'a', 'C'))
 		return SF_FORMAT_FLAC ;
 
-	/* Turtle Beach SMP 16-bit */
-	if (buffer [0] == MAKE_MARKER ('S', 'O', 'U', 'N') && buffer [1] == MAKE_MARKER ('D', ' ', 'S', 'A'))
-		return 0 ;
-
-	if (buffer [0] == MAKE_MARKER ('S', 'Y', '8', '0') || buffer [0] == MAKE_MARKER ('S', 'Y', '8', '5'))
-		return 0 ;
-
-	if (buffer [0] == MAKE_MARKER ('a', 'j', 'k', 'g'))
-		return 0 /*-SF_FORMAT_SHN-*/ ;
-
 	if (buffer [0] == MAKE_MARKER ('2', 'B', 'I', 'T'))
 		return SF_FORMAT_AVR ;
 
 	if (buffer [0] == MAKE_MARKER ('R', 'F', '6', '4') && buffer [2] == MAKE_MARKER ('W', 'A', 'V', 'E'))
 		return SF_FORMAT_RF64 ;
 
-	/* This must be the second last one. */
+	if (buffer [0] == MAKE_MARKER ('I', 'D', '3', 3))
+	{	psf_log_printf (psf, "Found 'ID3' marker.\n") ;
+		if (id3_skip (psf))
+			return guess_file_type (psf) ;
+		return 0 ;
+		} ;
+
+	/* Turtle Beach SMP 16-bit */
+	if (buffer [0] == MAKE_MARKER ('S', 'O', 'U', 'N') && buffer [1] == MAKE_MARKER ('D', ' ', 'S', 'A'))
+		return 0 ;
+
+	/* Yamaha sampler format. */
+	if (buffer [0] == MAKE_MARKER ('S', 'Y', '8', '0') || buffer [0] == MAKE_MARKER ('S', 'Y', '8', '5'))
+		return 0 ;
+
+	if (buffer [0] == MAKE_MARKER ('a', 'j', 'k', 'g'))
+		return 0 /*-SF_FORMAT_SHN-*/ ;
+
+	/* This must be the last one. */
 	if (psf->filelength > 0 && (format = try_resource_fork (psf)) != 0)
 		return format ;
 
@@ -2497,8 +2525,8 @@ psf_close (SF_PRIVATE *psf)
 	if (psf->peak_info)
 		free (psf->peak_info) ;
 
-	if (psf->broadcast_var)
-		free (psf->broadcast_var) ;
+	if (psf->broadcast_16k)
+		free (psf->broadcast_16k) ;
 
 	if (psf->loop_info)
 		free (psf->loop_info) ;
@@ -2628,7 +2656,7 @@ psf_open_file (SF_PRIVATE *psf, SF_INFO *sfinfo)
 		{	error = SFE_ZERO_MAJOR_FORMAT ;
 			goto error_exit ;
 			} ;
-		if ((SF_CONTAINER (psf->sf.format)) == 0)
+		if ((SF_CODEC (psf->sf.format)) == 0)
 		{	error = SFE_ZERO_MINOR_FORMAT ;
 			goto error_exit ;
 			} ;
@@ -2801,12 +2829,23 @@ psf_open_file (SF_PRIVATE *psf, SF_INFO *sfinfo)
 
 	/* For now, check whether embedding is supported. */
 	format = SF_CONTAINER (psf->sf.format) ;
-	if (psf->fileoffset > 0 &&
-			(format != SF_FORMAT_WAV) && (format != SF_FORMAT_WAVEX) &&
-			(format != SF_FORMAT_AIFF) && (format != SF_FORMAT_AU)
-			)
-	{	error = SFE_NO_EMBED_SUPPORT ;
-		goto error_exit ;
+	if (psf->fileoffset > 0)
+	{	switch (format)
+		{	case SF_FORMAT_WAV :
+			case SF_FORMAT_WAVEX :
+			case SF_FORMAT_AIFF :
+			case SF_FORMAT_AU :
+				/* Actual embedded files. */
+				break ;
+
+			case SF_FORMAT_FLAC :
+				/* Flac with an ID3v2 header? */
+				break ;
+
+			default :
+				error = SFE_NO_EMBED_SUPPORT ;
+				goto error_exit ;
+			} ;
 		} ;
 
 	if (psf->fileoffset > 0)
